function [Features, flag_others] = Features_normal_vs_others(ecg, fs)
%% Signal Normalization
signal = (ecg-mean(ecg))./std(ecg);

Features = [];

%% Baseline removal
signal_baseline_free = remove_baseline_ecg(signal, fs);
baseline_signal = signal - signal_baseline_free;

Features_baseline_signal = array_characteristics(baseline_signal);
Features = [Features Features_baseline_signal bandpower(baseline_signal)];
%%
[~, R] = pan_tompkin_algorithm_segmented(signal_baseline_free, fs);
R = R(R > 0);
R_peaks = unique(R);

[Q_ind, R_ind, S_ind]  = get_qrs_complex(signal_baseline_free, R_peaks, fs);

%% Amplitudes:
Q_amplitudes = signal_baseline_free(Q_ind);
R_amplitudes = signal_baseline_free(R_ind);
S_amplitudes = signal_baseline_free(S_ind);

RS_amplitude = R_amplitudes - S_amplitudes;
RQ_amplitude = R_amplitudes - Q_amplitudes;
qrs_width = S_ind - Q_ind;

Features_RS_amplitude = array_characteristics(RS_amplitude);
Features_RQ_amplitude = array_characteristics(RQ_amplitude);
Features_qrs_width = array_characteristics(qrs_width);

Features = [Features Features_RS_amplitude Features_RQ_amplitude Features_qrs_width];
%% HR features
RR_intervals = diff(R_peaks);
HR = 60*fs./RR_intervals;
HR_diff = diff(HR);
HR_diff_2 = diff(HR_diff);
HR_diff_3 = diff(HR_diff_2);

Features_HR = array_characteristics(HR);
Features_HR_diff =  array_characteristics(HR_diff);
Features_HR_diff_2 =  array_characteristics(HR_diff_2);
Features_HR_diff_3 =  array_characteristics(HR_diff_3);

Features = [Features Features_HR Features_HR_diff Features_HR_diff_2 Features_HR_diff_3];

%% Bradycardia test Find QRS sequence of 3 members with HR < 40 bpm - Number of times where this happens is our feature

HR_limit_down = 40;
bradycardia_feature = numel(find(HR <= HR_limit_down))/numel(HR);
Features = [Features bradycardia_feature];
%% Extreme bradycardia feature - Heart rate lower than 40 bpm for 5 consecutive beats

belowThreshold = [0, HR < HR_limit_down, 0];
edges = diff(belowThreshold);
rising = find(edges == 1);     %rising/falling edges
falling = find(edges == -1);  
spanWidth = falling - rising; 
wideEnough = spanWidth >= 5;   %% 5 consecutive values

if (isempty(find(wideEnough)))
    feature_extreme_bradycardia = 0;
else
    feature_extreme_bradycardia = 1;
end

Features = [Features feature_extreme_bradycardia];

%% Tachycardia test Find QRS sequence of 12 beats with HR > 130 bpm - Number of times where this happens is our feature
HR_limit_up = 130;
tachycardia_feature = numel(find(HR >= HR_limit_up))/numel(HR);

Features = [Features tachycardia_feature];

%% Extreme Tachycardia - Heart rate higher than 140 bpm for 17 consecutive beats

HR_limit_up_extreme = 140;

aboveThreshold = [0, HR > HR_limit_up_extreme, 0];
edges = diff(aboveThreshold);
rising = find(edges == 1);     %rising/falling edges
falling = find(edges == -1);  
spanWidth = falling - rising; 
min_element = min([17, numel(R_peaks)]);
wideEnough = spanWidth >= min_element;   

if (isempty(find(wideEnough)))
    feature_extreme_tachycardia = 0;
else
    feature_extreme_tachycardia = 1;
end

Features = [Features feature_extreme_tachycardia];

if (feature_extreme_tachycardia)
     flag_others = 1;
else
     flag_others = 0;
end

%% 

HR_down_1 = 46;
HR_down_2 = 40;

HR_up_1 = 100;
HR_up_2 = 130;

if (mean(HR) < HR_down_1)
    feature_down_1 = 1;
else 
    feature_down_1 = 0;
end

if (mean(HR) < HR_down_2)
    feature_down_2 = 1;
else 
    feature_down_2 = 0;
end

if (mean(HR) > HR_up_1)
    feature_up_1 = 1;
else 
    feature_up_1 = 0;
end

if (mean(HR) > HR_up_2)
    feature_up_2 = 1;
else 
    feature_up_2 = 0;
end

Features = [Features feature_down_1 feature_down_2 feature_up_1 feature_up_2];

%% number of times we have HR higher than 100 bpms!
HR_limit_up_new = 100;
tachycardia_feature_new = numel(find(HR >= HR_limit_up_new))/numel(HR);
Features = [Features tachycardia_feature_new];

%% Features for P wave - Normal sinus rhythm  0.12 < PR interval < 0.20 sec   -- Go from Q

Samples_from_Q_point = 0.01 * fs; 
duration_p_wave = 0.15 * fs; %% 120 ms < PR interval < 200 ms

sign_of_p_wave = [];
p_wave_peaks  = [];
PR_interval = [];

for i = 1 : length(Q_ind)
    
    index_end = Q_ind(i) - Samples_from_Q_point;
    index_start = index_end - duration_p_wave;
    index_start = round(index_start);
    index_end = round(index_end);
    
    if (index_start <= 0)
        continue
    else
        sig_p_wave = signal_baseline_free(index_start:index_end);
        [~, p_peak] = max(sig_p_wave);
        ind_p_peak = p_peak + index_start - 1;
        p_wave_peaks = [p_wave_peaks ind_p_peak];
        PR_interval = [PR_interval (Q_ind(i) - ind_p_peak)];
        sign_of_p_wave = [sign_of_p_wave sign(max(sig_p_wave))];  %% For junctional arrhythmia, P wave can be negative
        
        
    end
    
end


Features = [Features sum(sign_of_p_wave)/length(Q_ind) std(sign_of_p_wave)/length(Q_ind)];
%% PR interval -  %% 120 ms < PR interval < 200 ms

Features = [Features std(PR_interval)/std(RR_intervals)];
Features = [Features array_characteristics(PR_interval)];

%% PP interval 
PP_interval = diff(p_wave_peaks);

ratio_PP_RR_mean = mean(RR_intervals)/mean(PP_interval);
ratio_PP_RR_std = std(RR_intervals)/std(PP_interval);

Features = [Features ratio_PP_RR_mean ratio_PP_RR_std];
    
%% Premature ventricular or atrial contractions
R_ampl = ecg(R_peaks)./median(ecg(R_peaks));
percsmallPCs = sum(R_ampl<0.8)/length(R_ampl);
perchighPCs = sum(R_ampl>1.2)/length(R_ampl);

Features = [Features percsmallPCs perchighPCs];

%%  Normal sinus rhythm QT interval 0.35-0.43 secs
[~,~,~,delbeats, ~] = delineation_leadI_II(signal_baseline_free, fs);

QT_interval = (delbeats(:,8) - delbeats(:,4));
Features = [Features array_characteristics(QT_interval)];

QT_interval_secs = QT_interval/fs;
QT_interval_up = 0.8;

qt_above_limit = numel(find(QT_interval_secs > QT_interval_up))/numel(QT_interval_secs);

Features = [Features qt_above_limit];
%% Features for T wave - Ratio Rpeak/Twave

[~, ~, ~, delbeats, ~] = delineation_leadI_II(signal_baseline_free, fs);

Peak_T_wave = signal_baseline_free(delbeats(:,8));
Peak_R_wave =  signal_baseline_free(delbeats(:,5));

Ratio_R_T_amplitudes = Peak_R_wave./Peak_T_wave;

percent_t_above_r = numel(find(Ratio_R_T_amplitudes < 1))/numel(Peak_R_wave);
Features = [Features percent_t_above_r];

%% T wave similarities

q = 1;
n_right = 120;
offset = 20;

for n = 1 : length(S_ind)
    
    start_ind = S_ind(n) + offset;
    end_ind = S_ind(n) + n_right - 1;  
    
    if (start_ind <= 0 || end_ind <= 0)
        continue
    else
        if (end_ind >= length(signal_baseline_free) || start_ind >= length(signal_baseline_free))
            continue 
        else 
            pens_after(q,:) = signal_baseline_free(start_ind:end_ind);
            q = q + 1;
        end
    end
    
end

Average_t_wave = pens_after';

Features = [Features shanon_entropy(Average_t_wave(:))];

corr_matrix_t_wave = corr(Average_t_wave);

num_considered_t_waves = size(corr_matrix_t_wave, 1);

feature_corr_t_wave = (sum(corr_matrix_t_wave(:)) - num_considered_t_waves)/(num_considered_t_waves.^2 - num_considered_t_waves);

Features = [Features feature_corr_t_wave];

counter_crossings_t_wave = number_zerocrossings(Average_t_wave(:));

counter_crossings_t_wave = counter_crossings_t_wave/num_considered_t_waves;

Features = [Features counter_crossings_t_wave];

%% Concatenated T wave power features

T_wave_signal = Average_t_wave(:);
total_power_t_wave = bandpower(T_wave_signal);
f_low_t_wave = bandpower(T_wave_signal, fs, [0 5])/total_power_t_wave;
f_high_t_wave = bandpower(T_wave_signal, fs, [5 30])/total_power_t_wave;

Features = [Features f_low_t_wave f_high_t_wave];

sig_just_t_wave = medfilt1(signal_baseline_free, ceil(fs * 0.2));

Features = [Features shanon_entropy(sig_just_t_wave)];

%% Features for P wave

q = 1;
n_left = 140;
offset = 20;

for n = 1 : length(Q_ind)
    
    end_ind = Q_ind(n) - offset;
    start_ind = end_ind - n_left; 
    
    if (start_ind <= 0 || end_ind <= 0)
        continue
    else
        if (end_ind >= length(signal_baseline_free) || start_ind >= length(signal_baseline_free))
            continue 
        else 
            pens_before(q,:) = signal_baseline_free(start_ind:end_ind);
            q = q + 1;
        end
    end
    
end

Average_p_wave = pens_before';

Features = [Features shanon_entropy(Average_p_wave(:))];

corr_matrix_p_wave = corr(Average_p_wave);

num_considered_p_waves = size(corr_matrix_p_wave, 1);

feature_corr_p_wave = (sum(corr_matrix_p_wave(:)) - num_considered_p_waves)/(num_considered_p_waves.^2 - num_considered_p_waves);

Features = [Features feature_corr_p_wave];

counter_crossings_p_wave = number_zerocrossings(Average_p_wave(:));

counter_crossings_p_wave = counter_crossings_p_wave/num_considered_p_waves;

Features = [Features counter_crossings_p_wave];

 %%  Concatenated P wave power features 
P_wave_signal = Average_p_wave(:);

total_power_p_wave = bandpower(P_wave_signal);

f_low_p_wave = bandpower(P_wave_signal, fs, [0 30])/total_power_p_wave;

Features = [Features f_low_p_wave];

%% Morphology between 2 R peaks! for atrial flatter

[Q_ind_redef, R_ind_redef, S_ind_redef] = redefined_indexes(Q_ind, R_ind, S_ind);

QS_segment = Q_ind_redef(2:end) - S_ind_redef(1:end-1);

length_qs_interval = median(QS_segment);
length_qs_interval = fix(length_qs_interval);
indexes_to_remove_s = find(QS_segment < (length_qs_interval - 10));
indexes_to_remove_q = indexes_to_remove_s + 1;

S_new = setdiff(S_ind_redef, S_ind_redef(indexes_to_remove_s), 'stable');
Q_new = setdiff(Q_ind_redef, Q_ind_redef(indexes_to_remove_q), 'stable');

QS_segment_new =  Q_new(2:end) - S_new(1:end-1);

q = 1;

for i = 1:length(QS_segment_new)
    
    start_ind = S_new(i); 
    end_ind = S_new(i) + length_qs_interval -1;
    
     if (end_ind >= length(signal_baseline_free))
         continue
     else
         if (QS_segment_new(i) >= length_qs_interval)
            segment_qs = signal_baseline_free(start_ind:end_ind);
         else
             end_ind_new = start_ind + QS_segment_new(i) - 1; 
             value_ind_new = signal_baseline_free(end_ind_new);
             length_additionals = length_qs_interval - QS_segment_new(i);
             segment_qs = [signal_baseline_free(start_ind:end_ind_new); value_ind_new*ones(length_additionals, 1)];    
         end        
           Signal_qs_mor(q,:) = segment_qs;
           q = q + 1;
                
     end
                 
end


corr_matrix_signal_qs_mor = corr(Signal_qs_mor');

num_considered_qs_segments = size(corr_matrix_signal_qs_mor, 1);

feature_corr_qs_segment = (sum(corr_matrix_signal_qs_mor(:)) - num_considered_qs_segments)/(num_considered_qs_segments.^2 - num_considered_qs_segments);

Features = [Features feature_corr_qs_segment];

%% Morphology between 2 R peaks - number of zerocrossings

QS_zero_crossings = [];

for i = 1 : length(R_ind_redef) - 1
    
    start_ind = R_ind_redef(i); 
    end_ind = Q_ind_redef(i+1);
    
    if (end_ind >= length(signal_baseline_free))
         continue
    else
        QS_zero_crossings = [QS_zero_crossings number_zerocrossings(signal_baseline_free(start_ind:end_ind))];        
    end
    
end

Features = [Features mean(QS_zero_crossings)];


%% 
num_beats = 3;
max_min_HR = [];

for i = 1:length(HR)- num_beats
    array_HR = HR(i: i + num_beats);
    max_min_HR = [max_min_HR (max(array_HR) - min(array_HR))];
end

HR_max_change_1 = 5;
HR_max_change_2 = 10;

hr_group_change_1 = numel(find(max_min_HR > HR_max_change_1))/(length(HR)- num_beats);
hr_group_change_2 = numel(find(max_min_HR > HR_max_change_2))/(length(HR)- num_beats);

Features = [Features hr_group_change_1 hr_group_change_2];


%% HR group changes

num_beats = 5;
max_min_HR = [];

for i = 1:length(HR)- num_beats
    array_HR = HR(i: i + num_beats);
    max_min_HR = [max_min_HR (max(array_HR) - min(array_HR))];
end

HR_max_change_1 = 5;
HR_max_change_2 = 10;

hr_group_change_1 = numel(find(max_min_HR > HR_max_change_1))/(length(HR)- num_beats);
hr_group_change_2 = numel(find(max_min_HR > HR_max_change_2))/(length(HR)- num_beats);

Features = [Features hr_group_change_1 hr_group_change_2];

%%

num_beats = 5; 

HR_group = [];
Std_group = [];

num_beats = 5; 
for i = 1 : fix(length(HR)/num_beats)
    HR_group = HR(1 + (i-1)*num_beats : i*num_beats);
    Std_group = [Std_group std(HR_group)];
    
end

Features = [Features mean(Std_group) std(Std_group)];

%% Power features ECG signal

output = [];
rrdiff = RR_intervals;

[qrsfid, sign_variable, ~] = qrs_detect2(ecg);
qrsdiff = diff(qrsfid);
output = [output sum(sign_variable)];

[waveletfid] = delineation_r_peaks_wavelet(ecg,fs, 2);
waveletdiff = diff(waveletfid);

[fastfid] = fast_delineation(ecg, fs);
fastdiff = diff(fastfid);
output=[output length(waveletdiff)/length(rrdiff) length(fastdiff)/length(rrdiff) length(qrsdiff)/length(rrdiff)];

tmp=length(removeoutliers(rrdiff))/length(rrdiff);    
output = [output tmp];

tmp = length(rrdiff)/length(ecg);
output = [output tmp];

tmp = bandpower(ecg, fs, [0.005 0.02])/bandpower(ecg);
output=[output tmp];

tmp = bandpower(ecg, fs, [0.02 0.05])/bandpower(ecg);
output=[output tmp];

tmp = bandpower(ecg, fs, [0.05 40])/bandpower(ecg);
output=[output tmp];

tmp = bandpower(ecg, fs, [40 50])/bandpower(ecg);
output=[output tmp];

tmp = bandpower(ecg, fs, [50 100])/bandpower(ecg);
output=[output tmp];

tmp = bandpower(ecg, fs, [100 fs/2 - 10])/bandpower(ecg);
output=[output tmp];

[tmp1, tmp2] = snr(ecg);
output=[output tmp1 tmp2]; 

Features = [Features output];

%% T wave

dist_duration = mean(fs./RR_intervals)/3;
duration_t_wave = dist_duration * fs; 
Signals_t_wave = [];
Signals_t_wave_diff = [];
Signals_t_wave_diff_2 = [];
Signals_t_wave_diff_3 = [];

Signals_t_wave_norm = [];
Signals_t_wave_diff_norm = [];
Signals_t_wave_diff_2_norm = [];
Signals_t_wave_diff_3_norm = [];

Signals_t_wave_diff_sum = [];

for i = 1 : length(S_ind)
    
    index_start =  S_ind(i);
    index_end = index_start + duration_t_wave;
    index_end = round(index_end);
    
    if (index_end >= length(signal_baseline_free))
        continue
    else
        sig_t_wave = signal_baseline_free(index_start:index_end);
        sig_t_wave_diff = diff(sig_t_wave);
        sig_t_wave_diff_2 = diff(sig_t_wave_diff);
        sig_t_wave_diff_3 = diff(sig_t_wave_diff_2);
        
        Signals_t_wave = [Signals_t_wave sig_t_wave(:)];
        Signals_t_wave_diff = [Signals_t_wave_diff sig_t_wave_diff(:)];
        Signals_t_wave_diff_2 = [Signals_t_wave_diff_2 sig_t_wave_diff_2(:)];
        Signals_t_wave_diff_3 = [Signals_t_wave_diff_3 sig_t_wave_diff_3(:)];  
        
        Signals_t_wave_norm = [Signals_t_wave_norm norm(sig_t_wave, 2)];
        Signals_t_wave_diff_norm = [Signals_t_wave_diff_norm norm(sig_t_wave_diff, 2)];
        Signals_t_wave_diff_2_norm = [Signals_t_wave_diff_2_norm norm(sig_t_wave_diff_2, 2)];
        Signals_t_wave_diff_3_norm = [Signals_t_wave_diff_3_norm norm(sig_t_wave_diff_3, 2)];
        
        Signals_t_wave_diff_sum = [Signals_t_wave_diff_sum sum(sig_t_wave_diff)];
        
    end
    
end

Feat_t_1 = array_characteristics(min(Signals_t_wave));
Feat_t_2 = array_characteristics(max(Signals_t_wave));
Feat_t_3 = array_characteristics(mean(Signals_t_wave));
Feat_t_4 = array_characteristics(median(Signals_t_wave));
Feat_t_5 = array_characteristics(std(Signals_t_wave));

Features = [Features Feat_t_1 Feat_t_2 Feat_t_3 Feat_t_4 Feat_t_5];

Feat_t_6 = array_characteristics(min(Signals_t_wave_diff));
Feat_t_7 = array_characteristics(max(Signals_t_wave_diff));
Feat_t_8 = array_characteristics(mean(Signals_t_wave_diff));
Feat_t_9 = array_characteristics(median(Signals_t_wave_diff));
Feat_t_10 = array_characteristics(std(Signals_t_wave_diff));

Features = [Features Feat_t_6 Feat_t_7 Feat_t_8 Feat_t_9 Feat_t_10];

Feat_t_11 = array_characteristics(min(Signals_t_wave_diff_2));
Feat_t_12 = array_characteristics(max(Signals_t_wave_diff_2));
Feat_t_13 = array_characteristics(mean(Signals_t_wave_diff_2));
Feat_t_14 = array_characteristics(median(Signals_t_wave_diff_2));
Feat_t_15 = array_characteristics(std(Signals_t_wave_diff_2));

Features = [Features Feat_t_11 Feat_t_12 Feat_t_13 Feat_t_14 Feat_t_15];

Feat_t_16 = array_characteristics(min(Signals_t_wave_diff_3));
Feat_t_17 = array_characteristics(max(Signals_t_wave_diff_3));
Feat_t_18 = array_characteristics(mean(Signals_t_wave_diff_3));
Feat_t_19 = array_characteristics(median(Signals_t_wave_diff_3));
Feat_t_20 = array_characteristics(std(Signals_t_wave_diff_3));


Features = [Features Feat_t_16 Feat_t_17 Feat_t_18 Feat_t_19 Feat_t_20];

Feat_t_21 = array_characteristics(Signals_t_wave_norm);
Feat_t_22 = array_characteristics(Signals_t_wave_diff_norm);
Feat_t_23 = array_characteristics(Signals_t_wave_diff_2_norm);
Feat_t_24 = array_characteristics(Signals_t_wave_diff_3_norm);
Feat_t_25 = array_characteristics(Signals_t_wave_diff_sum);

Features = [Features Feat_t_21 Feat_t_22 Feat_t_23 Feat_t_24 Feat_t_25];


end

