function classifyResult = challenge(recordName)

% Sample entry for the 2017 PhysioNet/CinC Challenge.
%
% INPUTS:
% recordName: string specifying the record name to process
%
% OUTPUTS:
% classifyResult: integer value where
%                     N = normal rhythm
%                     A = AF
%                     O = other rhythm
%                     ~ = noisy recording (poor signal quality)
%
% To run your entry on the entire training set in a format that is
% compatible with PhysioNet's scoring enviroment, run the script
% generateValidationSet.m
%
% The challenge function requires that you have downloaded the challenge
% data 'training_set' in a subdirectory of the current directory.
%    http://physionet.org/physiobank/database/challenge/2017/
%
% This dataset is used by the generateValidationSet.m script to create
% the annotations on your training set that will be used to verify that
% your entry works properly in the PhysioNet testing environment.
%
%
% Version 1.0
%
%
% Written by: Chengyu Liu and Qiao Li January 20 2017
%             chengyu.liu@emory.edu  qiao.li@emory.edu
%
% Last modified by:
%
%

% load('parameters_trial.mat')
load('parameters_combined_scheme.mat')
%% AF determination
[tm, ecg, fs, siginfo]=rdmat(recordName);
Features = Feature_extraction_final(ecg, fs);
%%
Features = (Features - mu_norm)./sigma_norm;

y = predict(Mdl_multiclass_classifier, Features);

if (y == 1)
    Features_binary = Features_normal_vs_others(ecg, fs);
    Features_binary = (Features_binary - mu_norm_binary)./sigma_norm_binary;
    y_binary = str2double(predict(mdl_normal_others, Features_binary)); 
    if (y_binary == 1)
        classifyResult = 'N'; 
    else
        classifyResult = 'O';
    end
elseif (y == 2)
	   classifyResult = 'A'; 
elseif (y == 3)  
       classifyResult = 'O'; 
else 
       classifyResult = '~'; 
end

end


