function [qrs_amp_raw, Rloc] = pan_tompkin_algorithm_segmented(sig_short, fs)

%FRAN: I changed the threshold calculation with respect to the previous
%impementation. Before, the thresholds were calculated at the beginning and
%updated for every new peak. This caused problems when the signal was very
%noisy at some point, since the thresholds took ages to come back to a
%value where the peaks following the noise could be detected. Now the
%thresholds are calculated using 2 seconds of the signal every time a peak
%is detected

sig_short = sig_short - mean(sig_short);

%% Initialize
qrs_c =[]; %amplitude of R
qrs_i =[]; %index
SIG_LEV = 0;
skip = 0; % becomes one when a T wave is detected
m_selected_RR = 0;
mean_RR = 0;
qrs_i_raw =[];
qrs_amp_raw=[];
ser_back = 0;
test_m = 0;

%% Filtering - High Pass Filter & Low pass filter
% displayMessage('Filtering...');
%% Low Pass Filter  H(z) = ((1 - z^(-6))^2)/(1 - z^(-1))^2
b_lpf = [1 zeros(1, 5) -2 zeros(1,5) 1]/36; %try to find an automatic was for this
a_lpf = [1 -2 1];
[h_lpf, w_lpf] = freqz(b_lpf, a_lpf, 2000);
f_lpf = fs*w_lpf/(2*pi);
ecg_lpf = filter(b_lpf, a_lpf, sig_short);
ecg_lpf = ecg_lpf/ max(abs(ecg_lpf));
%% High Pass filter H(z) = (-1 + 32z^(-16)- 32z^(-17) + z^(-32))/(32 - 32z^(-1))
b_hpf = [-1 zeros(1, 15) 32 -32 zeros(1, 14) 1];
a_hpf = [32 -32];
ecg_hpf = filter(b_hpf, a_hpf, ecg_lpf);
ecg_hpf = ecg_hpf/ max(abs(ecg_hpf));
[h_hpf, w_hpf] = freqz(b_hpf, a_hpf, 2000);
f_hpf = fs*w_hpf/(2*pi);
%% derivative filter H(z) = 0.1(2 + z^(-1) - z^(-3) - 2z^(-4))
h_der = [2 1 0 -1 -2]*0.1;
ecg_der = filter(h_der, 1, ecg_hpf);
ecg_der = ecg_der/max(ecg_der);
%% Squaring nonlinearly enhance the dominant peaks
ecg_s = ecg_der.^2;
%% Moving average Y(nt) = (1/N)[x(nT-(N - 1)T)+ x(nT - (N - 2)T)+...+x(nT)]
% Namely in the paper we have a sampling frequency of 200Hz for which we
% have a moving window having 30 samples i.e. 150ms... So, 200:150 = 250:x
% -> we are getting a new length of our window which is equal to 187.5 ms
length_maw = 0.150;
b = ones(1 ,round(length_maw*fs))/round(length_maw*fs);
ecg_m = filter(b, 1, ecg_s);
%% Finding potential R peaks 
%displayMessage('Finding candidates...');
[pks,locs] = findpeaks(ecg_m, 'MinPeakDistance', round(0.3*fs));
%% initialize the training phase (2 seconds of the signal) to determine the THR_SIG and THR_NOISE
THR_SIG = max(ecg_m(1:2*fs))*1/3; % 0.25 of the max amplitude
THR_NOISE = mean(ecg_m(1:2*fs))*1/2; % 0.5 of the mean signal is considered to be noise
SIG_LEV= THR_SIG;
NOISE_LEV = THR_NOISE;
%% Initialize bandpath filter threshold(2 seconds of the bandpass signal)
THR_SIG1 = max(ecg_hpf(1:2*fs))*1/3; % 0.25 of the max amplitude
THR_NOISE1 = mean(ecg_hpf(1:2*fs))*1/2; %
SIG_LEV1 = THR_SIG1; % Signal level in Bandpassed filter
NOISE_LEV1 = THR_NOISE1; % Noise level in Bandpassed filter

%% Thresholding and online desicion rule
%displayMessage('Detecting R peaks...');
while length(ecg_m)-locs(end)<2*fs
    locs(end) = [];
    pks(end) = [];
end

for i = 1 : length(pks)
    
    THR_SIG = max(ecg_m(locs(i):locs(i)+2*fs))*1/3; % 0.25 of the max amplitude
    THR_NOISE = mean(ecg_m(locs(i):locs(i)+2*fs))*1/2; % 0.5 of the mean signal is considered to be noise
    SIG_LEV= THR_SIG;
    NOISE_LEV = THR_NOISE;
    %% Initialize bandpath filter threshold(2 seconds of the bandpass signal)
    THR_SIG1 = max(ecg_hpf(locs(i):locs(i)+2*fs))*1/3; % 0.25 of the max amplitude
    THR_NOISE1 = mean(ecg_hpf(locs(i):locs(i)+2*fs))*1/2; %
    SIG_LEV1 = THR_SIG1; % Signal level in Bandpassed filter
    NOISE_LEV1 = THR_NOISE1; % Noise level in Bandpassed filter
    %% locate the corresponding peak in the filtered signal - the output of the HPF ecg_hpf
    if locs(i)-round(length_maw*fs)>= 1 && locs(i)<= length(ecg_hpf)
        [y_i x_i] = max(ecg_hpf(locs(i)-round(length_maw*fs):locs(i)));
    else
        if i == 1
            [y_i x_i] = max(ecg_hpf(1:locs(i)));
            ser_back = 1;
        elseif locs(i)>= length(ecg_hpf)
            [y_i x_i] = max(ecg_hpf(locs(i)-round(length_maw*fs):end));
        end
        
    end
    
    %% update the heart_rate (Two heart rate means one the most recent and the other selected)
    if length(qrs_c) >= 9      
        diffRR = diff(qrs_i(end-8:end)); %calculate RR interval
        mean_RR = mean(diffRR); % calculate the mean of 8 previous R waves interval
        comp =qrs_i(end)-qrs_i(end-1); %latest RR
        if comp <= 0.92*mean_RR || comp >= 1.16*mean_RR
            % lower down thresholds to detect better in MVI
            THR_SIG = 0.5*(THR_SIG);
            % lower down thresholds to detect better in Bandpass filtered
            THR_SIG1 = 0.5*(THR_SIG1);
            %THR_NOISE1 = 0.5*(THR_SIG1); 
        else
            m_selected_RR = mean_RR; %the latest regular beats mean
        end        
    end
    
    %% calculate the mean of the last 8 R waves to make sure that QRS is not
    % missing(If no R detected , trigger a search back) 1.66*mean
    
    if m_selected_RR
        test_m = m_selected_RR; %if the regular RR availabe use it
    elseif mean_RR && m_selected_RR == 0
        test_m = mean_RR;
    else
        test_m = 0;
    end
    
    if test_m
        if (locs(i) - qrs_i(end)) >= round(1.66*test_m)% it shows a QRS is missed
            [pks_temp, locs_temp] = max(ecg_m(qrs_i(end)+ round(0.200*fs):locs(i)-round(0.200*fs))); % search back and locate the max in this interval
            locs_temp = qrs_i(end)+ round(0.200*fs) + locs_temp -1; %location
            
            if pks_temp > THR_NOISE
                qrs_c = [qrs_c pks_temp];
                qrs_i = [qrs_i locs_temp];
                
                % find the location in filtered sig
                if locs_temp <= length(ecg_hpf)
                    [y_i_t x_i_t] = max(ecg_hpf(locs_temp-round(length_maw*fs):locs_temp));
                else
                    [y_i_t x_i_t] = max(ecg_hpf(locs_temp-round(length_maw*fs):end));
                end
                % take care of bandpass signal threshold
                if y_i_t > THR_NOISE1
                    
                    qrs_i_raw = [qrs_i_raw locs_temp-round(length_maw*fs)+ (x_i_t - 1)];% save index of bandpass
                    qrs_amp_raw =[qrs_amp_raw y_i_t]; %save amplitude of bandpass
                    SIG_LEV1 = 0.25*y_i_t + 0.75*SIG_LEV1; %when found with the second thres
                end
                
                SIG_LEV = 0.25*pks_temp + 0.75*SIG_LEV ;  %when found with the second threshold
            end
        end
    end
    
    
    %find noise and QRS peaks
    if pks(i) >= THR_SIG
        
        % if a QRS candidate occurs within 360ms of the previous QRS
        % ,the algorithm determines if its T wave or QRS
        if length(qrs_c) >= 3
            if (locs(i)-qrs_i(end)) <= round(0.36*fs)
                Slope1 = mean(diff(ecg_m(locs(i)-round(0.075*fs):locs(i)))); %mean slope of the waveform at that position
                Slope2 = mean(diff(ecg_m(qrs_i(end)-round(0.075*fs):qrs_i(end)))); %mean slope of previous R wave
                if abs(Slope1) <= abs(0.5*(Slope2))  % slope less then 0.5 of previous R
                    skip = 1; % T wave identification
                    % adjust noise level in both filtered and
                    % MVI
                    NOISE_LEV1 = 0.125*y_i + 0.875*NOISE_LEV1;
                    NOISE_LEV = 0.125*pks(i) + 0.875*NOISE_LEV;
                else
                    skip = 0;
                end
                
            end
        end
        
        if skip == 0  % skip is 1 when a T wave is detected
            qrs_c = [qrs_c pks(i)];
            qrs_i = [qrs_i locs(i)];
            
            % bandpass filter check threshold
            if y_i >= THR_SIG1
                if ser_back
                    qrs_i_raw = [qrs_i_raw x_i];  % save index of bandpass
                else
                    qrs_i_raw = [qrs_i_raw locs(i)-round(length_maw*fs)+ (x_i - 1)];% save index of bandpass
                end
                qrs_amp_raw =[qrs_amp_raw y_i];% save amplitude of bandpass
                SIG_LEV1 = 0.125*y_i + 0.875*SIG_LEV1;% adjust threshold for bandpass filtered sig
            end
            
            % adjust Signal level
            SIG_LEV = 0.125*pks(i) + 0.875*SIG_LEV ;
        end
        
        
    elseif THR_NOISE <= pks(i) && pks(i)<THR_SIG
        
        %adjust Noise level in filtered sig
        NOISE_LEV1 = 0.125*y_i + 0.875*NOISE_LEV1;
        %adjust Noise level in MVI
        NOISE_LEV = 0.125*pks(i) + 0.875*NOISE_LEV;
        
    elseif pks(i) < THR_NOISE
        
        % noise level in filtered signal
        NOISE_LEV1 = 0.125*y_i + 0.875*NOISE_LEV1;
        %end
        
        %adjust Noise level in MVI
        NOISE_LEV = 0.125*pks(i) + 0.875*NOISE_LEV;
    end
    
    %adjust the threshold with SNR
    if NOISE_LEV ~= 0 || SIG_LEV ~= 0
        THR_SIG = NOISE_LEV + 0.25*(abs(SIG_LEV - NOISE_LEV));
        THR_NOISE = 0.5*(THR_SIG);
    end
    
    % adjust the threshold with SNR for bandpassed signal
    if NOISE_LEV1 ~= 0 || SIG_LEV1 ~= 0
        THR_SIG1 = NOISE_LEV1 + 0.25*(abs(SIG_LEV1 - NOISE_LEV1));
        THR_NOISE1 = 0.5*(THR_SIG1);
    end
    
    skip = 0; %reset parameters
    ser_back = 0;  %reset bandpass param
    

end

qrs_i_raw = qrs_i_raw-22;


%%

signal_last_part = sig_short(qrs_i_raw(end) : end);
index  = delineation_wavelet(signal_last_part, fs);
index = index + qrs_i_raw(end);

qrs_i_raw = [qrs_i_raw index];



%% Removing outliers 


num = 20;

Rloc = [];

for t = 1: length(qrs_i_raw)
    
    ind_R_1 = qrs_i_raw(t);
    
    if (ind_R_1 - num >= 1)
        
        if (ind_R_1 + num >= length(sig_short))
            final_index = length(sig_short);
        else 
            final_index = ind_R_1 + num;
        end
        
        [~, ind] = max(sig_short(ind_R_1 - num : final_index));
        Rloc = [Rloc (ind_R_1 - num - 1 + ind)];
        
    else
        
        continue
    
    end
    
    
end



end