function [ indOE,twOE,tpOE,indchanged ] = triangWave_OnOff( fs,final_sig,ind_wind,tw_wind,tp_wind )
%TRIANGWAVE_ONOFF This function finds onset and offset of a wave
%considering the peak and the isoelectric line. When the euclidean distance
%between the signal and the segment peak-onset(offset) and the one between
%the signal and the isoelectric line is minimum, then the onset(offset)
%considered is the point to chose. For ECG delineation. 

% INPUTS
% sig_wind: the window of the signal to consider. We take the isoelectril
% line in this window;
% peaks: the indices and the type wave of the peak to consider to determine
% the onset and offset (for an ECG, it can include P and T peaks)

% OUTPUTS
% indOE,twOE,tpOE : onset and offset (in this order) of
% the wave considered as index, type of the wave (p for ex.) and type of
% the point (onset or offset)
indOE = [];
twOE = [];
tpOE = [];
indchanged = [];

R = ind_wind(tw_wind==0 & tp_wind==1);
T = ind_wind(tw_wind==2  & tp_wind==1);
P = ind_wind(tw_wind==1 & tp_wind==1);
Q = ind_wind(tw_wind==0 & tp_wind==0);
qrs_dur = round(fs*0.08);
pwavehalf_dur = round(fs*0.10/2);
twavehalf_dur = round(fs*0.19/2);
for b = 2:length(R)
    % Consider T and P between 2 R peaks
    tb = T(find(T>=R(b-1),1,'first'));
    pb = P(find(P<=R(b),1,'last'));
    qb = Q(find(Q<=R(b),1,'last'));
    if(isempty(pb) || pb<tb)
        indchanged = [indchanged pb];
        pb = R(b)-qrs_dur-pwavehalf_dur; 
        indOE = [indOE pb]; 
        twOE = [twOE 1]; 
        tpOE = [tpOE 1];
    end

    % Find the isoelectric line considering the signal between the T wave and
    % the next P wave. For each RR beat
    isoline = isoelectric_line(fs,final_sig,tb,pb);

    % Find onset and offset - Consider the peak and assume that the origin
    % of the reference system is on the isoelectric line centered on the
    % peak
    if(b==2 && tb-36<=0)
        maxintTB = tb-2;
    else
        maxintTB = 35;
    end
    
    %Parameters done for the noise
    if(~isempty(qb) && qb>pb)
        maxintPBoff = qb-pb;
    else
        maxintPBoff = pwavehalf_dur+5;
    end
    maxintPBon = pwavehalf_dur+5;
    
    
    
    % P offset and onset
    [~,pbmaxoff] = min(abs(final_sig(pb:pb+maxintPBoff)-isoline));
    [~,pbmaxon] = min(abs(final_sig(pb:-1:pb-maxintPBon)-isoline));
    [ indpOffset ] = min_eucldist_isotriang( final_sig(pb:pb+pbmaxoff),isoline,0,final_sig,pb,0 );
    [ indpOnset ] = min_eucldist_isotriang( final_sig(pb:-1:pb-pbmaxon),isoline,1,final_sig,pb,0 );
%     [ indpOffset ] = min_eucldist_isotriangcross( final_sig(pb:pb+pbmaxoff),isoline,0 );
%     [ indpOnset ] = min_eucldist_isotriangcross( final_sig(pb:-1:pb-pbmaxon),isoline,1 );
    pon = pb - indpOnset;
    poff = pb + indpOffset;
    if(isempty(pon) && ~isempty(poff))
        poff = pb+pwavehalf_dur;
    elseif(~isempty(pon) && isempty(poff))
        pon = pb-pwavehalf_dur;
    end
    indOE = [indOE pon poff];
    twOE = [twOE 1 1];
    tpOE = [tpOE 0 2];

    % T offset and onset    
    [~,tbmaxoff] = min(abs(final_sig(tb:tb+maxintTB)-isoline));
    [~,tbmaxon] = min(abs(final_sig(tb:-1:tb-maxintTB)-isoline));
    [ indtOffset ] = min_eucldist_isotriang( final_sig(tb:tb+tbmaxoff),isoline,0,final_sig,tb,1 );
    [ indtOnset ] = min_eucldist_isotriang( final_sig(tb:-1:tb-tbmaxon),isoline,1,final_sig,tb,1 );
%     [ indtOffset ] = min_eucldist_isotriangcross( final_sig(tb:tb+tbmaxoff),isoline,0 );
%     [ indtOnset ] = min_eucldist_isotriangcross( final_sig(tb:-1:tb-tbmaxon),isoline,1 );
    ton = tb - indtOnset;
    toff = tb + indtOffset;
    if(isempty(ton) && ~isempty(toff))
        toff = tb + twavehalf_dur;
    elseif(~isempty(ton) && isempty(toff))
        ton = tb - twavehalf_dur;
    end
    indOE = [indOE ton toff];
    twOE = [twOE 2 2];
    tpOE = [tpOE 0 2];
    
end


end

